"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var maybe_1 = require("./maybe");
var chai = require("chai");
var expect = chai.expect;
describe('Maybe', function () {
    var add1 = function (n) { return n + 1; };
    describe('isJust', function () {
        it('should return false when Nothing is provided', function () {
            expect(maybe_1.isJust(maybe_1.nothing())).to.be.false;
        });
        it('should return true when Just is provided', function () {
            expect(maybe_1.isJust(maybe_1.just('hola'))).to.be.true;
        });
        it('should return false when null is provided', function () {
            expect(maybe_1.isJust(null)).to.be.false;
        });
        it('should return false when undefined is provided', function () {
            expect(maybe_1.isJust(undefined)).to.be.false;
        });
    });
    describe('isNothing', function () {
        it('should return false when Just is provided', function () {
            expect(maybe_1.isNothing(maybe_1.just('hola'))).to.be.false;
        });
        it('should return true when Nothing is provided', function () {
            expect(maybe_1.isNothing(maybe_1.nothing())).to.be.true;
        });
        it('should return true when null is provided', function () {
            expect(maybe_1.isNothing(null)).to.be.true;
        });
        it('should return true when undefined is provided', function () {
            expect(maybe_1.isNothing(undefined)).to.be.true;
        });
    });
    describe('withDefault', function () {
        it('should return the Just value when Just is provided', function () {
            expect(maybe_1.withDefault(maybe_1.just(6), 0)).to.equal(6);
        });
        it('should return the default value when Nothing is provided', function () {
            expect(maybe_1.withDefault(maybe_1.nothing(), 0)).to.equal(0);
        });
        it('should return the default value when null is provided', function () {
            expect(maybe_1.withDefault(null, 1)).to.equal(1);
        });
        it('should return the default value when undefined is provided', function () {
            expect(maybe_1.withDefault(undefined, 2)).to.equal(2);
        });
    });
    describe('map', function () {
        it('should return Just when mapping over Just', function () {
            var maybeFour = maybe_1.just(4);
            var maybeFive = maybe_1.map(add1, maybeFour);
            expect(maybe_1.isJust(maybeFive)).to.be.true;
        });
        it('should return Nothing when mapping over Nothing', function () {
            var result = maybe_1.map(add1, maybe_1.nothing());
            expect(maybe_1.isNothing(result)).to.be.true;
        });
        it('should be isNothing === true when mapping over null', function () {
            var result = maybe_1.map(add1, null);
            expect(maybe_1.isNothing(result)).to.be.true;
        });
        it('should be isNothing === true when mapping over undefined', function () {
            var result = maybe_1.map(add1, undefined);
            expect(maybe_1.isNothing(result)).to.be.true;
        });
        it('should return 5 when adding 1 to Just(5)', function () {
            var maybeFour = maybe_1.just(4);
            var maybeFive = maybe_1.map(add1, maybeFour);
            expect(maybe_1.withDefault(maybeFive, 0)).to.equal(5);
        });
    });
    describe('map2', function () {
        var fullname = function (a, b) { return a + ' ' + b; };
        it('should return the fullname when appending Just("name") to Just("surname")', function () {
            var maybeName = maybe_1.just('John');
            var maybeSurame = maybe_1.just('Doe');
            var maybeFullname = maybe_1.map2(fullname, maybeName, maybeSurame);
            expect(maybe_1.withDefault(maybeFullname, '-')).to.equal('John Doe');
        });
        it('should return Nothing when appending Just() with Nothing', function () {
            var maybeName = maybe_1.just('John');
            var maybeFullname = maybe_1.map2(fullname, maybeName, maybe_1.nothing());
            expect(maybe_1.isNothing(maybeFullname)).to.be.true;
        });
        it('another example with numbers', function () {
            var safeParseInt = function (num) {
                var n = parseInt(num, 10);
                return isNaN(n) ? maybe_1.nothing() : maybe_1.just(n);
            };
            var sum = function (x, y) { return x + y; };
            var safeStringSum = function (x, y) {
                return maybe_1.map2(sum, safeParseInt(x), safeParseInt(y));
            };
            expect(safeStringSum('1', '2')).to.deep.equal(maybe_1.just(3));
            expect(safeStringSum('a', '2')).to.deep.equal(maybe_1.nothing());
        });
    });
    [maybe_1.map3, maybe_1.map4, maybe_1.map5].forEach(function (fn, index) {
        var n = index + 3;
        describe("map" + n, function () {
            var gt = function () {
                var nums = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    nums[_i] = arguments[_i];
                }
                return nums.every(function (num) { return num > 3; });
            };
            var just4Array = function (len) {
                return Array.apply(null, Array(len)).map(function () { return maybe_1.just(4); });
            };
            it('should return the true when all are greater than 3', function () {
                var result = fn.apply(void 0, __spreadArrays([gt], just4Array(n)));
                expect(maybe_1.withDefault(result, false)).to.be.true;
            });
            it('should be Nothing when any is Nothing', function () {
                var arr = just4Array(n);
                arr[2] = maybe_1.nothing();
                var result = fn.apply(void 0, __spreadArrays([gt], arr));
                expect(maybe_1.isNothing(result)).to.be.true;
            });
        });
    });
    describe('mapN', function () {
        var fullname = function (a, b, c) { return a + ' ' + b + ' ' + c; };
        it('should return the fullname when appending Just("name") to Just("middle") to Just("surname")', function () {
            var maybeName = maybe_1.just('John');
            var maybeMiddleName = maybe_1.just('D');
            var maybeSurame = maybe_1.just('Doe');
            var maybeFullname = maybe_1.mapN(fullname, maybeName, maybeMiddleName, maybeSurame);
            expect(maybe_1.withDefault(maybeFullname, '-')).to.equal('John D Doe');
        });
        it('should return Nothing when appending Just() with Nothing', function () {
            var maybeName = maybe_1.just('John');
            var maybeSurame = maybe_1.just('Doe');
            var maybeFullname = maybe_1.mapN(fullname, maybeName, maybe_1.nothing(), maybeSurame);
            expect(maybe_1.isNothing(maybeFullname)).to.be.true;
        });
    });
    describe('andThen', function () {
        var head = function (arr) { return (arr.length > 0 ? maybe_1.just(arr[0]) : maybe_1.nothing()); };
        it('should return Just(computation result) when Just is provided as argument', function () {
            var result = maybe_1.andThen(head, maybe_1.just(['a', 'b', 'c']));
            expect(maybe_1.withDefault(result, '')).to.equal('a');
        });
        it('should return Nothing when Nothing is provided as argument', function () {
            var result = maybe_1.andThen(head, maybe_1.nothing());
            expect(maybe_1.withDefault(result, '')).to.equal('');
        });
    });
    describe('caseOf', function () {
        it('should Launch 5 missiles', function () {
            var result = maybe_1.caseOf({
                Nothing: function () { return 'zzz'; },
                Just: function (n) { return "Launch " + n + " missiles"; }
            }, maybe_1.just('5'));
            expect(result).to.equal('Launch 5 missiles');
        });
        it('should zzz', function () {
            var result = maybe_1.caseOf({
                Nothing: function () { return 'zzz'; },
                Just: function (n) { return "Launch " + n + " missiles"; }
            }, maybe_1.nothing());
            expect(result).to.equal('zzz');
        });
    });
});
